import { produce } from 'sveltekit-sse';
import * as can from 'socketcan';
import {
	type RawMessage,
	marshalMessage,
	CAN_ID_AMS_ERROR,
	CAN_ID_AMS_SLAVE_STATUS_MASK,
	CAN_ID_AMS_SLAVE_LOG_MASK,
	CAN_ID_AMS_SLAVE_LOG_BASE,
	CAN_ID_AMS_SLAVE_STATUS_BASE,
	CAN_ID_AMS_STATUS,
	CAN_ID_SHUNT_CURRENT,
	CAN_ID_SHUNT_VOLTAGE_1,
	CAN_ID_SHUNT_VOLTAGE_2
} from '$lib/messages';

export function POST() {
	return produce(async function start({ emit }) {
		const network = can.createRawChannel('can0');
		network.setRxFilters([
			{ id: CAN_ID_AMS_ERROR, mask: 0xfff },
			{ id: CAN_ID_AMS_STATUS, mask: 0xfff },
			{ id: CAN_ID_AMS_SLAVE_STATUS_BASE, mask: CAN_ID_AMS_SLAVE_STATUS_MASK },
			{ id: CAN_ID_AMS_SLAVE_LOG_BASE, mask: CAN_ID_AMS_SLAVE_LOG_MASK },
			{ id: CAN_ID_SHUNT_CURRENT, mask: 0xfff },
			{ id: CAN_ID_SHUNT_VOLTAGE_1, mask: 0xfff },
			{ id: CAN_ID_SHUNT_VOLTAGE_2, mask: 0xfff }
		]);

		network.addListener('onMessage', (msg: RawMessage) => {
			emit('message', JSON.stringify(marshalMessage(msg)));
		});
		network.start();

		return function stop() {
			network.stop();
		};
	});
}
